/*
 * Copyright © 2004-2021 L2J Server
 * 
 * This file is part of L2J Server.
 * 
 * L2J Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * L2J Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.l2jserver.gameserver.data.xml.impl;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.l2jserver.gameserver.model.StatsSet;
import com.l2jserver.gameserver.model.actor.instance.L2PcInstance;
import com.l2jserver.gameserver.model.actor.transform.Transform;
import com.l2jserver.gameserver.model.actor.transform.TransformLevelData;
import com.l2jserver.gameserver.model.actor.transform.TransformTemplate;
import com.l2jserver.gameserver.model.holders.AdditionalItemHolder;
import com.l2jserver.gameserver.model.holders.AdditionalSkillHolder;
import com.l2jserver.gameserver.model.holders.SkillHolder;
import com.l2jserver.gameserver.network.serverpackets.ExBasicActionList;
import com.l2jserver.gameserver.util.IXmlReader;

/**
 * Transformation data.
 * @author UnAfraid
 */
public final class TransformData implements IXmlReader {
	
	private static final Logger LOG = LoggerFactory.getLogger(TransformData.class);
	
	private final Map<Integer, Transform> _transformData = new HashMap<>();
	
	protected TransformData() {
		load();
	}
	
	@Override
	public synchronized void load() {
		_transformData.clear();
		parseDatapackDirectory("data/stats/transformations", false);
		LOG.info("Loaded {} transform templates.", _transformData.size());
	}
	
	@Override
	public void parseDocument(Document doc) {
		for (Node n = doc.getFirstChild(); n != null; n = n.getNextSibling()) {
			if ("list".equalsIgnoreCase(n.getNodeName())) {
				for (Node d = n.getFirstChild(); d != null; d = d.getNextSibling()) {
					if ("transform".equalsIgnoreCase(d.getNodeName())) {
						NamedNodeMap attrs = d.getAttributes();
						StatsSet set = new StatsSet();
						for (int i = 0; i < attrs.getLength(); i++) {
							Node att = attrs.item(i);
							set.set(att.getNodeName(), att.getNodeValue());
						}
						final Transform transform = new Transform(set);
						for (Node cd = d.getFirstChild(); cd != null; cd = cd.getNextSibling()) {
							boolean isMale = "Male".equalsIgnoreCase(cd.getNodeName());
							if ("Male".equalsIgnoreCase(cd.getNodeName()) || "Female".equalsIgnoreCase(cd.getNodeName())) {
								TransformTemplate templateData = null;
								for (Node z = cd.getFirstChild(); z != null; z = z.getNextSibling()) {
									switch (z.getNodeName()) {
										case "common" -> {
											for (Node s = z.getFirstChild(); s != null; s = s.getNextSibling()) {
												switch (s.getNodeName()) {
													case "base", "stats", "defense", "magicDefense", "collision", "moving" -> {
														attrs = s.getAttributes();
														for (int i = 0; i < attrs.getLength(); i++) {
															Node att = attrs.item(i);
															set.set(att.getNodeName(), att.getNodeValue());
														}
													}
												}
											}
											templateData = new TransformTemplate(set);
											transform.setTemplate(isMale, templateData);
										}
										case "skills" -> {
											if (templateData == null) {
												templateData = new TransformTemplate(set);
												transform.setTemplate(isMale, templateData);
											}
											for (Node s = z.getFirstChild(); s != null; s = s.getNextSibling()) {
												if ("skill".equals(s.getNodeName())) {
													attrs = s.getAttributes();
													int skillId = parseInteger(attrs, "id");
													int skillLevel = parseInteger(attrs, "level");
													templateData.addSkill(new SkillHolder(skillId, skillLevel));
												}
											}
										}
										case "actions" -> {
											if (templateData == null) {
												templateData = new TransformTemplate(set);
												transform.setTemplate(isMale, templateData);
											}
											set.set("actions", z.getTextContent());
											final int[] actions = set.getIntArray("actions", " ");
											templateData.setBasicActionList(new ExBasicActionList(actions));
										}
										case "additionalSkills" -> {
											if (templateData == null) {
												templateData = new TransformTemplate(set);
												transform.setTemplate(isMale, templateData);
											}
											for (Node s = z.getFirstChild(); s != null; s = s.getNextSibling()) {
												if ("skill".equals(s.getNodeName())) {
													attrs = s.getAttributes();
													int skillId = parseInteger(attrs, "id");
													int skillLevel = parseInteger(attrs, "level");
													int minLevel = parseInteger(attrs, "minLevel");
													templateData.addAdditionalSkill(new AdditionalSkillHolder(skillId, skillLevel, minLevel));
												}
											}
										}
										case "items" -> {
											if (templateData == null) {
												templateData = new TransformTemplate(set);
												transform.setTemplate(isMale, templateData);
											}
											for (Node s = z.getFirstChild(); s != null; s = s.getNextSibling()) {
												if ("item".equals(s.getNodeName())) {
													attrs = s.getAttributes();
													int itemId = parseInteger(attrs, "id");
													boolean allowed = parseBoolean(attrs, "allowed");
													templateData.addAdditionalItem(new AdditionalItemHolder(itemId, allowed));
												}
											}
										}
										case "levels" -> {
											if (templateData == null) {
												templateData = new TransformTemplate(set);
												transform.setTemplate(isMale, templateData);
											}
											
											final StatsSet levelsSet = new StatsSet();
											for (Node s = z.getFirstChild(); s != null; s = s.getNextSibling()) {
												if ("level".equals(s.getNodeName())) {
													attrs = s.getAttributes();
													for (int i = 0; i < attrs.getLength(); i++) {
														Node att = attrs.item(i);
														levelsSet.set(att.getNodeName(), att.getNodeValue());
													}
												}
											}
											templateData.addLevelData(new TransformLevelData(levelsSet));
										}
									}
								}
							}
						}
						_transformData.put(transform.getId(), transform);
					}
				}
			}
		}
	}
	
	public Transform getTransform(int id) {
		return _transformData.get(id);
	}
	
	public boolean transformPlayer(int id, L2PcInstance player) {
		final Transform transform = getTransform(id);
		if (transform != null) {
			player.transform(transform);
		}
		return transform != null;
	}
	
	public static TransformData getInstance() {
		return SingletonHolder._instance;
	}
	
	private static class SingletonHolder {
		protected static final TransformData _instance = new TransformData();
	}
}
